#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "arc.h"

ARC_FILE *arc_open(char *filename)
{
	ARC_HEADER header;
	void *buf;
	ARC_FILE *af;
	FILE *f;
	unsigned int size;

	af = (ARC_FILE *)malloc( sizeof(ARC_FILE) );
	if( af == NULL )
		return NULL;

	f = fopen( filename, "rb" );

	if( f == NULL )
		return NULL;
	if( fread( &header, sizeof(ARC_HEADER), 1, f ) != 1 )
	{
		fclose( f );
		return NULL;
	}
	if( memcmp( header.magic, MAGIC, 4 ) != 0 ||
		header.size_of_fst == 0 )
	{
		fclose( f );
		return NULL;
	}
	if( fseek( f, se_32( header.fst_offset ), SEEK_SET ) != 0 )
	{
		fclose( f );
		return NULL;
	}	
	size = se_32( header.size_of_fst );

	if( size <= sizeof( ARC_FST_ENTRY ) )
	{
		fclose( f );
		return NULL;
	}

	buf = malloc( size );
	if( buf == NULL )
	{
		fclose( f );
		return 0;
	}
	if( fread( buf, size, 1, f ) != 1 )
	{
		free( buf );
		fclose( f );
		return NULL;
	}
	
	af->f = f;
	af->fst_data = buf;
	af->fst_data_size = size; //header.size_of_fst;

	return af;
}


void arc_close( ARC_FILE *af )
{
	if( af == NULL )
		return;
	if( af->f != NULL )
		fclose( af->f );
	if( af->fst_data != NULL )
		free( af->fst_data );

	free( af );

	return;
}


unsigned int arc_get_entry_count( ARC_FILE *af )
{
	ARC_FST_ENTRY *entry;

	if( af == NULL )
		return 0;
	
	entry = (ARC_FST_ENTRY *)af->fst_data;

	return( se_32( entry->entry_number ) );
}


unsigned int arc_find_first( ARC_FILE *af, ARC_ENTRY_DATA *ed )
{
	ARC_FST_ENTRY *fst;
	int maxentries = arc_get_entry_count( af );
	unsigned int info;

	if( ed == NULL ||
		af == NULL ||
		maxentries == 0 ||
		af->fst_data_size < 2 * sizeof( ARC_FST_ENTRY )
		)
		return 0;

	fst = (ARC_FST_ENTRY *)af->fst_data;

	//skip "root"
	fst++;

	info = se_32( fst->info );
	ed->entry_number = se_32( fst->entry_number );
	ed->file_offset = 0; // unknown, currently
	ed->data_size = 0; // unknown, currently
	ed->name = (char *)(af->fst_data) + maxentries * sizeof(ARC_FST_ENTRY) + (info & 0xFFFFFF);
	ed->type = ( (info & 0xFF000000) != 0 );

	return 1;
}

int arc_get_file_content( ARC_FILE *af, unsigned int fileoffset, size_t size, void *buffer )
{
	if( af == NULL || af->f == NULL || buffer == NULL )
		return 0;

	if( fread( buffer, size, 1, af->f ) != 1 )
		return 0;

	return 1;
}

unsigned int arc_find_next( ARC_FILE *af, unsigned int n, ARC_ENTRY_DATA *ed )
{
	ARC_FST_ENTRY *fst;
	int maxentries = arc_get_entry_count( af );
	unsigned int info;
	unsigned int current = n;

	if( ed == NULL ||
		af == NULL ||
		maxentries == 0 ||
		af->fst_data_size < 2 * sizeof( ARC_FST_ENTRY ) ||
		++current > maxentries-1
		)
		return 0;

	fst = (ARC_FST_ENTRY *)af->fst_data;

	// "seek" to next file
	fst += current;

	info = se_32( fst->info );
	ed->data_size = se_32( fst->entry_number );
	ed->file_offset = se_32( fst->file_offset );
	ed->name = (char *)(af->fst_data) + maxentries * sizeof(ARC_FST_ENTRY) + (info & 0xFFFFFF);
	ed->type = ( (info & 0xFF000000) != 0 );

	return current;
}