#ifndef _ARC_H
#define _ARC_H

#define MAGIC "\x55\xAA\x38\x2D"

typedef struct
{
	FILE *f;
	void *fst_data;
	size_t fst_data_size;
} ARC_FILE;

typedef struct
{
	unsigned char magic[4]; // 0x55,0xAA,0x38,0x2D
	unsigned int fst_offset; // file offset of file structure table
	unsigned int size_of_fst; // size of file structure table
	unsigned int unknown; 
	unsigned int unknown2[4];
} ARC_HEADER;

typedef struct
{
	unsigned int info; // type(t) OR offset(f) -> (unsigned int)ttffffff
					   // types: (0 = file, 1 = directory)
					   // offset: this is an offset into an array of
					   // zero-terminated strings
					   // (its base address is arc_header.fst_offset +
					   // the first ARC_FST_ENTRY's entry_number * sizeof(ARC_FST_ENTRY))
	unsigned int file_offset; // file offset if type == 0
							  // set to 1 if type == 1
	unsigned int entry_number;
} ARC_FST_ENTRY;

typedef struct
{
	unsigned int entry_number;
	size_t data_size;
	unsigned char type;
	char *name;
	char *path;
	unsigned int file_offset;
} ARC_ENTRY_DATA;



ARC_FILE *arc_open(char *filename);
void arc_close( ARC_FILE *af );
unsigned int arc_get_entry_count( ARC_FILE *af );
unsigned int arc_find_first( ARC_FILE *af, ARC_ENTRY_DATA *ed );
int arc_get_file_content( ARC_FILE *af, unsigned int fileoffset, size_t size, void *buffer );

__inline unsigned int se_32(unsigned int v)
{
	__asm
	{
		mov eax, v
		bswap eax
	}
}

#endif